package com.hxstrive.spring_security.config;

import jakarta.servlet.ServletException;
import jakarta.servlet.http.HttpServletRequest;
import jakarta.servlet.http.HttpServletResponse;
import org.springframework.context.annotation.Bean;
import org.springframework.context.annotation.Configuration;
import org.springframework.security.config.annotation.web.builders.HttpSecurity;
import org.springframework.security.config.annotation.web.configurers.AbstractAuthenticationFilterConfigurer;
import org.springframework.security.core.Authentication;
import org.springframework.security.web.SecurityFilterChain;
import org.springframework.security.web.authentication.AuthenticationSuccessHandler;
import org.springframework.security.web.authentication.logout.LogoutHandler;
import org.springframework.security.web.authentication.logout.LogoutSuccessHandler;

import java.io.IOException;

import static org.springframework.security.config.Customizer.withDefaults;

/**
 * Spring Security 配置类
 * @author hxstrive.com
 */
@Configuration
public class SecurityConfig {

    @Bean
    public SecurityFilterChain securityFilterChain(HttpSecurity http) throws Exception {
        http.authorizeHttpRequests(authorize -> authorize.anyRequest().authenticated())
                // formLogin 用于配置表单登录功能。
                // permitAll() 表示允许所有用户访问表单登录相关的页面，例如登录页面、登录处理接口等。
                .formLogin(e ->
                        // 设置登录成功自动跳转到 /success 页面，true 表示自动重定向到该页面
                        e.defaultSuccessUrl("/success", true).permitAll())
//                .logout(e -> e.logoutUrl("/logout")
//                        .logoutSuccessUrl("/login?logout=success")
//                )
                .logout(e -> e.logoutUrl("/logout")
                        // 添加自定义退出逻辑
                        .addLogoutHandler(new LogoutHandler() {
                            @Override
                            public void logout(HttpServletRequest request, HttpServletResponse response,
                                               Authentication authentication) {
                                System.out.println(authentication.getName() + " 已退出");
                                // 清理自定义缓存或资源
                                // 其他操作
                            }
                        })
                        // 自定义退出成功处理逻辑
                        .logoutSuccessHandler(new LogoutSuccessHandler(){
                            @Override
                            public void onLogoutSuccess(HttpServletRequest request, HttpServletResponse response,
                                                        Authentication authentication) throws IOException, ServletException {
                                // 登录成功后重定向到 /login?logout=success 页面
                                response.sendRedirect("/login?logout=success");
                            }
                        })
                )
                // 用于启用 HTTP 基本认证，withDefaults() 表示使用默认的 HTTP 基本认证配置。
                .httpBasic(withDefaults());

        return http.build();
    }

}
